"""Test configuration module for Vandamme Proxy tests."""

from .test_config import (
    DEFAULT_TEST_CONFIG,
    TEST_API_KEYS,
    TEST_ENDPOINTS,
    TEST_HEADERS,
    TEST_MODELS,
)

__all__ = [
    "TEST_API_KEYS",
    "TEST_ENDPOINTS",
    "DEFAULT_TEST_CONFIG",
    "TEST_HEADERS",
    "TEST_MODELS",
]
